@@ -7,7 +7,7 @@
 // or the GNU General Public License version 2 or later
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: WordResults.cc,v 1.3 2001/06/29 14:14:08 loic Exp $
+// $Id: WordResults.cc,v 1.4 2001/07/20 10:37:53 loic Exp $
 //
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -156,6 +156,34 @@ int WordResults::Open(char* namep)
     return NOTOK;
   }
 
+  //
+  // Uniq list
+  //
+  if((error = CDB_db_create(&uniq, dbenv, 0)) != 0) {
+    fprintf(stderr, "WordResults::Open db_create(%s) uniq failed: %s\n", name, CDB_db_strerror(error));
+    return NOTOK;
+  }
+
+  if((error = uniq->set_pagesize(uniq, WORD_RESULT_PAGESIZE)) != 0) {
+    fprintf(stderr, "WordResults::Open uniq->set_pagesize(%d) failed: %s\n", WORD_RESULT_PAGESIZE, CDB_db_strerror(error));
+    return NOTOK;
+  }
+
+  if((error = uniq->set_bt_compare(uniq, wordResults_cmp)) != 0) {
+    fprintf(stderr, "WordResults::Open uniq->set_bt_compare(%s) failed: %s\n", name, CDB_db_strerror(error));
+    return NOTOK;
+  }
+  
+  if((error = uniq->open(uniq, (char*)filename, "uniq", DB_BTREE, DB_CREATE, 0666)) != 0) {
+    fprintf(stderr, "WordResults::Open uniq->open(%s) failed: %s\n", name, CDB_db_strerror(error));
+    return NOTOK;
+  }
+
+  if((error = uniq->cursor(uniq, 0, &uniq_cursor, 0)) != 0) {
+    fprintf(stderr, "WordResults::Open uniq->cursor(%s) failed: %s\n", name, CDB_db_strerror(error));
+    return NOTOK;
+  }
+
   //
   // Ranked list
   //
@@ -238,6 +266,18 @@ int WordResults::Close()
     return NOTOK;
   }
 
+  //
+  // Uniq list
+  //
+  if((error = uniq_cursor->c_close(uniq_cursor)) != 0) {
+    fprintf(stderr, "WordResults::Close uniq_cursor->close(%s) failed: %s\n", name, CDB_db_strerror(error));
+  }
+  
+  if((error = uniq->close(uniq, 0)) != 0) {
+    fprintf(stderr, "WordResults::Close uniq->close(%s) failed: %s\n", name, CDB_db_strerror(error));
+    return NOTOK;
+  }
+
   //
   // Ranked list
   //
@@ -274,6 +314,7 @@ int WordResults::Close()
 
   name = 0;
   sorted = 0;
+  uniq = 0;
   ranked = 0;
   info = 0;
   dbenv = 0;
@@ -284,6 +325,7 @@ int WordResults::KeySemantic(const WordKeySemantic& nkey_semantic)
 {
   key_semantic = nkey_semantic;
 
+  uniq_offset = key_semantic.Uniq();
   document_offset = key_semantic.DocumentOffset();
   document_length = key_semantic.DocumentLength();
 
@@ -328,6 +370,33 @@ int WordResults::Exists(const WordKey& key) const
   return error == 0;
 }
 
+int WordResults::UniqExists(const WordKey& key) const
+{
+  DBT rkey;
+  memset((void*)&rkey, '\0', sizeof(DBT));
+  rkey.data = (void*)(key.Values() + uniq_offset);
+  rkey.size = sizeof(WordKeyNum);
+  rkey.app_private = (void*)1;
+
+  DBT rdata;
+  db_recno_t recno;
+  memset((void*)&rdata, '\0', sizeof(DBT));
+  rdata.data = (void*)&recno;
+  rdata.size = sizeof(db_recno_t);
+
+  int error;
+  if((error = uniq_cursor->c_get(uniq_cursor, &rkey, &rdata, DB_SET)) != 0) {
+    if(error != DB_NOTFOUND) {
+      fprintf(stderr, "WordResults::UniqExists uniq_cursor->c_get(%s) failed: %s\n", (char*)key.Get(), CDB_db_strerror(error));
+    }
+  }
+
+  if(verbose)
+    fprintf(stderr, "WordResults::UniqExists: %s %s\n", (char*)key.Get(), (error ? "was not found" : "was found"));
+
+  return error == 0;
+}
+
 int WordResults::Get(WordMatches* matches, unsigned int length, unsigned int position) const
 {
   if(verbose)
@@ -505,6 +574,25 @@ int WordResults::Put(const WordMatch& match, unsigned int position)
     return NOTOK;
   }
 
+  //
+  // Uniq list
+  //
+  if(uniq_offset) {
+    memset((void*)&rkey, '\0', sizeof(DBT));
+    rkey.data = (void*)(match.match.Values() + uniq_offset);
+    rkey.size = sizeof(WordKeyNum);
+    rkey.app_private = (void*)1;
+
+    memset((void*)&rdata, '\0', sizeof(DBT));
+    rdata.data = (void*)&recno;
+    rdata.size = sizeof(db_recno_t);
+
+    if((error = uniq_cursor->c_put(uniq_cursor, &rkey, &rdata, DB_KEYLAST)) != 0) {
+      fprintf(stderr, "WordResults::Put uniq_cursor->c_put(%s) failed: %s\n", (char*)match.Get(), CDB_db_strerror(error));
+      return NOTOK;
+    }
+  }
+
   return OK;
 }
 
