@@ -5,7 +5,7 @@
 // or the GNU General Public License version 2 or later
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: WordTree.cc,v 1.10 2001/06/29 14:14:08 loic Exp $
+// $Id: WordTree.cc,v 1.11 2001/07/20 10:37:53 loic Exp $
 //
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -37,7 +37,12 @@ int WordTree::WalkNextExclude(const WordKey& key)
 {
   int exclude = 0;
   if(results) {
-    exclude = results->Exists(key);
+    if(has_uniq) {
+      exclude = results->UniqExists(key);
+    }
+    if(!exclude) {
+      exclude = results->Exists(key);
+    }
   }
   return exclude;
 }
@@ -108,12 +113,32 @@ int WordTreeLiteral::WalkRewind()
 
 int WordTreeLiteral::WalkNext()
 {
-  int ret = WordCursorOne::WalkNext();
+  int ret;
+  int too_low = 0;
+
+  do {
+    ret = WordCursorOne::WalkNext();
+    //
+    // If the returned document is before the lowest document allowed,
+    // Seek and try again.
+    //
+    if(ret == OK &&
+       !bottom_document.Empty() &&
+       key_semantic.DocumentCompare(bottom_document, GetDocument()) > 0) {
+      WordKey patch = bottom_document;
+      key_semantic.RealmCopy(GetFound().Key(), patch);
+      ret = Seek(patch);
+      too_low = 1;
+    } else {
+      too_low = 0;
+    }
+  } while(ret == OK && too_low);
+
   if(verbose > 3) fprintf(stderr, "WordTreeLiteral::WalkNext: reached %s\n", (char*)GetDocument().Get());
 
   if(ret == OK && !top_document.Empty()) {
     if(key_semantic.DocumentCompare(top_document, GetDocument()) <= 0)
-      ret = WORD_WALK_ATEND;
+      ret = WORD_WALK_ATEND|WORD_WALK_ATEND_NOMATCH;
   }
 
   return ret;
@@ -392,7 +417,7 @@ int WordTreeOptional::WalkNext()
   // Advance cursors so that next call fetches another constraint
   //
   if(cursor_get_flags == DB_NEXT)
-    key_semantic.DocumentNext(constraint, uniq);
+    key_semantic.DocumentNext(constraint, has_uniq);
     
   if((ret = Seek(constraint)) != OK)
     return ret;
@@ -696,7 +721,7 @@ int WordTreeOptional::SearchCursorNot(WordTree& cursor, WordKey& document)
       // The cursor does not give any hint on a possible
       // next document, just go to the next possible one.
       //
-      key_semantic.DocumentNext(document, uniq);
+      key_semantic.DocumentNext(document, has_uniq);
       //
       // Since the current document changed, start over.
       //
@@ -755,7 +780,7 @@ int WordTreeOptional::SearchCursorNot(WordTree& cursor, WordKey& document)
       // The cursor does not give any hint on a possible
       // next document, just go to the next possible one.
       //
-      key_semantic.DocumentNext(document, uniq);
+      key_semantic.DocumentNext(document, has_uniq);
       //
       // Since the current document changed, start over.
       //
@@ -1028,7 +1053,7 @@ int WordTreeOr::WalkNext()
       // Advance cursors so that next call fetches another constraint
       //
       if(cursor_get_flags == DB_NEXT)
-	  key_semantic.DocumentNext(constraint, uniq);
+	  key_semantic.DocumentNext(constraint, has_uniq);
     
       if((ret = Seek(constraint)) != OK)
 	  return ret;
