@@ -74,7 +74,7 @@
 // or the GNU General Public License version 2 or later
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: mifluzsearch.cc,v 1.14 2001/06/29 14:14:08 loic Exp $
+// $Id: mifluzsearch.cc,v 1.15 2001/07/20 10:37:53 loic Exp $
 //
 
 #ifdef HAVE_CONFIG_H
@@ -425,15 +425,12 @@ static int dosearch(params_t* params)
 	exit(0);
       }
       
-      if(params->uniq_server) {
-	int uniq = config.Value("wordlist_wordkey_uniq", 0);
-	if(!uniq) {
-	  fprintf(stderr, "wordlist_wordkey_uniq not set or 0, -S is useless\n");
-	}
-	params->uniq_server = uniq;
+      int uniq = config.Value("wordlist_wordkey_uniq", 0);
+      if(params->uniq_server && !uniq) {
+	fprintf(stderr, "wordlist_wordkey_uniq not set or 0, -S is useless\n");
       }
 
-      WordTreeArg arg(words, params->uniq_server, params->proximity, document, document_length, location);
+      WordTreeArg arg(words, uniq, params->uniq_server, params->proximity, document, document_length, location);
       if(expr->Prepare(arg) != OK)
 	return -1;
     } else {
@@ -473,13 +470,19 @@ static int dosearch(params_t* params)
   //
   // Set lower and higher bounds if appropriate
   //
+  String low_string;
+  String high_string;
   {
     WordKey low(context);
     WordKey high(context);
-    if(params->low[0])
+    if(params->low[0]) {
       low.Set(params->low);
-    if(params->high[0])
+      low_string << "L" << params->low;
+    }
+    if(params->high[0]) {
       high.Set(params->high);
+      high_string << "H" << params->high;
+    }
     if(params->low[0] || params->high[0])
       if(expr->Bounds(low, high) != OK)
 	return -1;
@@ -511,7 +514,11 @@ static int dosearch(params_t* params)
     if(search->SetResults(results) != OK)
       return -1;
 
-    if(results->Open(String("Cmifluz_") + expr->search) != OK)
+    if(results->Open(String("Cmifluz_") +
+		     expr->search +
+		     String(params->uniq_server ? "U" : "") +
+		     low_string +
+		     high_string) != OK)
       return -1;
   }
   //
