@@ -16,12 +16,12 @@
 
 //
 // Part of the ht://Dig package   <http://www.htdig.org/>
-// Copyright (c) 1999, 2000 The ht://Dig Group
+// Copyright (c) 1999, 2000, 2001 The ht://Dig Group
 // For copyright details, see the file COPYING in your distribution
 // or the GNU General Public License version 2 or later
 // <http://www.gnu.org/copyleft/gpl.html>
 //
-// $Id: search_parser.cxx,v 1.6 2000/12/20 12:30:31 loic Exp $
+// $Id: search_parser.cxx,v 1.9 2001/06/29 14:14:08 loic Exp $
 //
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -133,7 +133,7 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-    73,    79,   102,   103,   112,   122,   123,   125,   138,   142
+    73,    79,   116,   117,   126,   136,   137,   139,   153,   157
 };
 #endif
 
@@ -740,20 +740,34 @@ case 2:
 {
 	  MifluzSearchInput* input = (MifluzSearchInput*)parm;
 	  if(yyval.node == 0) {
+	    /*
+	     * Single node is kept as is
+	     */
+	    yyval.node = yyvsp[0].node;
+	  } else if(yyval.node->IsA() == WORD_TREE_OR ||
+		    yyval.node->IsA() == WORD_TREE_OPTIONAL) {
+	    /*
+	     * Additional nodes are pushed in OR or OPTIONAL because it exists.
+	     */
+	    WordTreeOperand* op_or = (WordTreeOperand*)yyval.node;
+	    op_or->cursors = (WordTree**)realloc(op_or->cursors, sizeof(WordTree*) * (op_or->cursors_length + 1));
+	    op_or->cursors[op_or->cursors_length] = yyvsp[0].node;
+	    op_or->cursors_length++;
+	    yyval.node = op_or;
+	  } else {
+	    /*
+	     * Additional node implies we build an OR or OPTIONAL node to hold
+	     * the node collection.
+	     */
 	    WordTreeOperand* op_or;
 	    if(input->or_method == WORD_SEARCH_OR)
 	      op_or = new WordTreeOr(input->words, "");
 	    else
 	      op_or = new WordTreeOptional(input->words, "");
-	    op_or->cursors = (WordTree**)malloc(sizeof(WordTree*));
-	    op_or->cursors_length = 1;
-	    op_or->cursors[0] = yyvsp[0].node;
-	    yyval.node = op_or;
-	  } else {
-	    WordTreeOperand* op_or = (WordTreeOperand*)yyval.node;
-	    op_or->cursors = (WordTree**)realloc(op_or->cursors, sizeof(WordTree*) * (op_or->cursors_length + 1));
-	    op_or->cursors[op_or->cursors_length] = yyvsp[0].node;
-	    op_or->cursors_length++;
+	    op_or->cursors = (WordTree**)malloc(sizeof(WordTree*) * 2);
+	    op_or->cursors_length = 2;
+	    op_or->cursors[0] = yyval.node; /* The existing node */
+	    op_or->cursors[1] = yyvsp[0].node; /* The new node */
 	    yyval.node = op_or;
 	  }
 	  input->query = yyval.node;
@@ -788,6 +802,7 @@ case 8:
 	    WordTreeNear* near = new WordTreeNear(input->words, "");
 	    near->cursors = yyvsp[-1].list->values;
 	    near->cursors_length = yyvsp[-1].list->values_length;
+	    near->proximity = 1;
 	    yyval.node = near;
 	  }
 	;
